require 'spec_helper'

describe Crowdkit::Client::Jobs do
  context "jobs" do
    context "search" do
      let!(:request_stub) { stub_get(/jobs/).to_return(fixture_response("jobs/search.json")) }

      context "requests" do
        it "should be made" do
          client.jobs.search
          expect(request_stub).to have_been_requested
        end

        it "sets search parameter" do
          client.jobs.search("Rad")
          expect(a_request(:get, /jobs/).with(query: {"query" => "Rad"})).to have_been_made
        end

        it "passes along other parameters" do
          client.jobs.search(page: 2, state: :finished)
          expect(a_request(:get, /jobs/).with(query: {"page" => "2", "state" => "finished"})).to have_been_made
        end

        it "has list alias" do
          client.jobs.list
          expect(request_stub).to have_been_requested
        end

        it "has all alias" do
          client.jobs.all
          expect(request_stub).to have_been_requested
        end
      end

      context "responses" do
        it "parses the data" do
          expect(client.jobs.search.length).to eq(5)
        end

        it "executes a block" do
          i = 0
          client.jobs.all {|job| i += 1 }
          expect(i).to eq(5)
        end
      end
    end

    context "get" do
      let!(:request_stub) { stub_get("/jobs/1").to_return(fixture_response("jobs/job.json")) }

      context "request" do
        it "should be made" do
          client.jobs.get(1)
          expect(request_stub).to have_been_requested
        end

        it "accepts job parameter" do
          client.jobs(1).get
          expect(request_stub).to have_been_requested
        end

        it "blows up without id" do
          expect { client.jobs.get }.to raise_error Crowdkit::ValidationError
        end

        it "has find alias" do
          client.jobs.find(1)
          expect(request_stub).to have_been_requested
        end
      end

      context "response" do
        it "parses" do
          expect(client.jobs.get(1).id).to eq(1)
        end

        it "forwards to data" do
          expect(client.jobs.get(1).title).to eq("Some really awesome job that is so very very cool")
        end
      end
    end

    context "copy" do
      let!(:request_stub) { stub_post(/11\/copy/).to_return(fixture_response("jobs/copy.json", 202)) }

      context "request" do
        it "should be made" do
          client.jobs(11).copy
          expect(request_stub).to have_been_requested
        end

        it "passes parameters" do
          client.jobs(11).copy(all_units: true)
          expect(a_request(:post, /copy/).with(query: {"all_units" => "true"})).to have_been_made
        end

        it "blows up without id" do
          expect { client.jobs.copy }.to raise_error Crowdkit::ValidationError
        end
      end

      context "response" do
        it "parses" do
          expect(client.jobs.copy(11).id).to eq(12)
        end

        it "is a 202" do
          client.jobs(11).copy(all_units: true)
          expect(client.last_response.status).to eq(202)
        end

        it "provides current status" do
          status = stub_get("/statuses/3d05fc4a1501dcebfb836434d8fc5cce").to_return(fixture_response("statuses/status.json"))
          response = client.jobs(11).copy(all_units: true)
          response.current_status
          response.current_status
          expect(status).to have_been_requested.twice
        end
      end
    end

    context "order" do
      let!(:request_stub) { stub_post(/11\/order/).to_return(fixture_response("jobs/order.json", 202)) }

      context "request" do
        it "should be made" do
          client.jobs(11).order
          expect(request_stub).to have_been_requested
        end

        it "passes parameters" do
          client.jobs(11).order(units_count: 10)
          expect(a_request(:post, /order/).with(query: {"units_count" => "10"})).to have_been_made
        end

        it "blows up without id" do
          expect { client.jobs.order }.to raise_error Crowdkit::ValidationError
        end
      end

      context "response" do
        it "parses" do
          expect(client.jobs.order(11).id).to eq(1822)
        end

        it "is a 202" do
          client.jobs.order(units_count: 10, job_id: 11)
          expect(client.last_response.status).to eq(202)
        end

        it "provides current status" do
          status = stub_get("/statuses/9e61a51bad819441e3d5fcfc6045b555").to_return(fixture_response("statuses/status.json"))
          response = client.jobs(11).order
          response.current_status
          response.current_status
          expect(status).to have_been_requested.twice
        end
      end
    end

    context "create" do
      let!(:request_stub) { stub_post("/jobs").to_return(fixture_response("jobs/job.json", 201)) }

      context "request" do
        it "should be made" do
          client.jobs.create(title: "My amazing job")
          expect(request_stub).to have_been_requested
        end

        it "passes title in the body" do
          client.jobs.create(title: "My amazing job")
          expect(a_request(:post, /jobs/).with(body: {title: "My amazing job"}.to_json)).to have_been_made
        end

        it "blows up without title" do
          expect { client.jobs.create }.to raise_error Crowdkit::ValidationError
        end
      end

      context "response" do
        it "parses" do
          expect(client.jobs.create(title: "My amazing job").id).to eq(1)
        end

        it "is a 201" do
          client.jobs.create(title: "My amazing job")
          expect(client.last_response.status).to eq(201)
        end
      end
    end

    describe "statuses" do
      let!(:request_stub) { stub_get("/jobs/11/statuses").to_return(fixture_response("statuses/index.json")) }

      describe "request" do
        it "should be made" do
          client.jobs(11).statuses
          expect(request_stub).to have_been_requested
        end

        it "blows up without id" do
          expect { client.jobs.statuses }.to raise_error Crowdkit::ValidationError
        end
      end

      describe "response" do
        it "parses" do
          expect(client.jobs(11).statuses.length).to eq(2)
        end
      end
    end

    describe "judgments" do
      let(:get_judgments) { client.jobs(11).judgments(per_page: 3) }
      let!(:request_stub) { stub_get("/jobs/11/judgments?per_page=3").to_return(fixture_response("judgments/index.json")) }

      describe "request" do
        it "should be made" do
          get_judgments
          expect(request_stub).to have_been_requested
        end

        it "passes parameters" do
          get_judgments
          expect(a_request(:get, /judgments/).with(query: {"per_page" => 3.to_s})).to have_been_made
        end

        it "blows up without id" do
          expect { client.jobs.judgments }.to raise_error Crowdkit::ValidationError
        end
      end

      describe "response" do
        it "parses" do
          expect(get_judgments.length).to eq(3)
        end
      end
    end

    context "namespaces" do
      describe "units" do
        it "provides association" do
          expect(client.jobs.units.class).to eq(Crowdkit::Client::Units)
        end

        it "passes along job_id" do
          expect(client.jobs(1).units.stored_params).to eq({job_id: 1})
        end
      end
    end

    describe "upload" do
      let(:file) { fixture('upload.csv') }
      let!(:request_stub) { stub_put('/jobs/1/upload').to_return(fixture_response('jobs/job.json', 201)) }

      describe "request" do
        it "should be made" do
          client.jobs(1).upload(file)
          expect(request_stub).to have_been_requested
        end

        it "blows up without id" do
          expect { client.jobs.upload(file) }.to raise_error Crowdkit::ValidationError
        end
      end

      context "response" do
        it "parses" do
          expect(client.jobs(1).upload(file).id).to eq(1)
          expect(client.last_response.status).to eq(201)
        end
      end
    end
  end
end
