require 'spec_helper'

describe Crowdkit::Client::Units do
  context "show" do
    let(:unit_id) { 47979 }
    let!(:request_stub) { stub_get("/units/#{unit_id}").to_return(fixture_response("units/show.json")) }

    context "requests" do
      it "gets made" do
        client.units.get(unit_id)
        expect(request_stub).to have_been_requested
      end

      it "has find alias" do
        client.units.find(unit_id)
        expect(request_stub).to have_been_requested
      end

      it "blows up without id" do
        expect { client.units.get }.to raise_error Crowdkit::ValidationError
      end
    end

    context "response" do
      it "parses" do
        expect(client.units.get(unit_id).id).to eq(unit_id)
      end

      specify "hypermedia" do
        expect(client.units.get(unit_id).rels.to_hash).to eq({
          self_url: "https://api.crowdflower.dev/v2/units/47979",
          judgments_url: "https://api.crowdflower.dev/v2/units/47979/judgments"
        })
      end
    end
  end

  describe "list" do
    let!(:request_stub) { stub_get("/jobs/10/units").to_return(fixture_response("units/index.json")) }

    context "requests" do
      it "gets made" do
        client.units(job_id: 10).list
        expect(request_stub).to have_been_requested
      end

      it "has all alias" do
        client.units.all(job_id: 10)
        expect(request_stub).to have_been_requested
      end

      it "blows up without job_id" do
        expect { client.units.list }.to raise_error Crowdkit::ValidationError
      end
    end

    context "response" do
      it "parses" do
        expect(client.units(job_id: 10).list.length).to eq(50)
      end
    end
  end

  context "create" do
    let(:data) { Hash(data: {test: true}) }
    let(:create_unit) { client.units(job_id: 10).create(data.dup) }
    let!(:request_stub) { stub_post("/jobs/10/units").to_return(fixture_response("units/show.json", 201)) }

    context "request" do
      it "should be made" do
        create_unit
        expect(request_stub).to have_been_requested
      end

      it "passes data in the body" do
        create_unit
        expect(a_request(:post, /jobs/).with(body: data.to_json)).to have_been_made
      end

      it "blows up without job_id" do
        expect { client.units.create }.to raise_error Crowdkit::ValidationError
      end

      it "support array of units data" do
        units_data = [{test_1: true}, {test_2: true}]
        client.units(job_id: 10).create(units_data)
        expect(a_request(:post, /jobs\/10\/units/).with(body: units_data.to_json)).to have_been_made
      end
    end

    context "response" do
      it "parses" do
        expect(create_unit.id).to eq(47979)
      end

      it "is a 201" do
        create_unit
        expect(client.last_response.status).to eq(201)
      end
    end
  end

  context "copy" do
    let(:unit_id) { 47979 }
    let(:destination_job_id) { 11 }
    let(:copy_unit) { client.units(unit_id).copy(destination_job_id: destination_job_id) }
    let!(:request_stub) do
      url = "/units/#{unit_id}/copy?destination_job_id=#{destination_job_id}"
      stub_post(url).to_return(fixture_response("units/copy.json", 201))
    end

    context "request" do
      it "should be made" do
        copy_unit
        expect(request_stub).to have_been_requested
      end

      it "passes parameters" do
        copy_unit
        expect(a_request(:post, /copy/).with(query: {"destination_job_id" => destination_job_id.to_s})).to have_been_made
      end

      it "blows up without id" do
        expect { client.units(unit_id).copy }.to raise_error Crowdkit::ValidationError
      end
    end

    context "response" do
      it "parses" do
        expect(copy_unit.data[:job][:id]).to eq(destination_job_id)
      end

      it "is a 201" do
        copy_unit
        expect(client.last_response.status).to eq(201)
      end
    end
  end

  context "delete" do
    let!(:request_stub) { stub_delete("/units/1").to_return(body: '', status: 204) }
    before(:each) { client.units(1).delete }

    it "request should be made" do
      expect(request_stub).to have_been_requested
    end

    it "response status is 204" do
      expect(client.last_response.status).to eq(204)
    end
  end

  context "pause" do
    let!(:request_stub) { stub_put("/units/1/pause").to_return(body: '', status: 204) }
    before(:each) { client.units(1).pause }

    it "request should be made" do
      expect(request_stub).to have_been_requested
    end

    it "response status is 204" do
      expect(client.last_response.status).to eq(204)
    end
  end

  context "resume" do
    let!(:request_stub) { stub_put("/units/1/resume").to_return(body: '', status: 204) }
    before(:each) { client.units(1).resume }

    it "request should be made" do
      expect(request_stub).to have_been_requested
    end

    it "response status is 204" do
      expect(client.last_response.status).to eq(204)
    end
  end

  describe "judgments" do
    let(:get_judgments) { client.units(1).judgments(per_page: 3) }
    let!(:request_stub) { stub_get("/units/1/judgments?per_page=3").to_return(fixture_response("judgments/index.json")) }

    describe "request" do
      it "should be made" do
        get_judgments
        expect(request_stub).to have_been_requested
      end

      it "passes parameters" do
        get_judgments
        expect(a_request(:get, /judgments/).with(query: {"per_page" => 3.to_s})).to have_been_made
      end

      it "blows up without id" do
        expect { client.units.judgments }.to raise_error Crowdkit::ValidationError
      end
    end

    describe "response" do
      it "parses" do
        expect(get_judgments.length).to eq(3)
      end
    end
  end

  describe "poll" do
    let!(:request_stub) do
      stub_get("/jobs/10/units/queue").to_return(fixture_response("units/poll/normal.json"))
    end

    describe "request" do
      it "should be made" do
        client.units(job_id: 10).poll
        expect(request_stub).to have_been_requested
      end

      it "should handle 304" do
        remove_request_stub(request_stub)
        stub_get("/jobs/10/units/queue").to_return({status: 304})
        result = client.jobs(10).units.poll
        expect(result.any?).to be_falsey
      end

      it "passes parameters" do
        stub_get("/jobs/10/units/queue?delete=true&limit=10").to_return({status: 304})
        client.units(job_id: 10).poll(delete: true, limit: 10)
        expect(a_request(:get, /queue/).with(query: {"delete" => "true", "limit" => "10"})).to have_been_made
      end

      it "blows up without job_id" do
        expect { client.units.poll }.to raise_error Crowdkit::ValidationError
      end

      describe "delete" do
        let!(:request_delete_stub) { stub_delete(/queue/).to_return({status: 204}) }
        it "should delete all units" do
          client.units(job_id: 10).poll.delete
          expect(request_delete_stub).to have_been_requested
        end

        it "should delete units in block" do
          client.units(job_id: 10).poll.each do |unit|
            unit.delete if unit.id == 47979
          end
          expect(request_delete_stub).to have_been_requested
        end

        it 'should delete single unit' do
          unit = client.units(job_id: 10).poll.first
          unit.delete
          expect(request_delete_stub).to have_been_requested
        end
      end
    end

    describe "response" do
      it "parses" do
        poll = client.units(job_id: 10).poll
        expect(poll.length).to eq(30)
        expect(poll.map(&:id).length).to eq(30)
        expect(poll.first.id).to eq(47979)
        expect(poll.last.id).to eq(48008)
      end
    end
  end
end
