require 'spec_helper'

describe Crowdkit do
  context "configuration" do
    context "defaults" do
      specify "connection_options" do
        expect(Crowdkit.config.connection_options).to include({
          headers: {
            accept: "application/json",
            user_agent: "Crowdkit Ruby Gem #{Crowdkit::VERSION}"
          }
        })
      end
    end

    context "stack" do
      specify "default" do
        expect(Crowdkit.config.stack.handlers.first).to eq(Faraday::Request::TokenAuthentication)
      end

      context "overrides" do
        before(:each) do
          Crowdkit.reset!
        end

        it "modifies existing stack" do
          Crowdkit.configure do |c|
            c.stack.insert_before(Faraday::Request::TokenAuthentication, Crowdkit)
          end
          expect(Crowdkit.config.stack.handlers.first).to eq Crowdkit
        end

        it "overrides stack" do
          Crowdkit.configure do |c|
            c.stack do |builder|
              builder.use Crowdkit
            end
          end
          expect(Crowdkit.config.stack.handlers.first).to eq Crowdkit
        end

        it "use custom adapter" do
          Crowdkit.configure(adapter: :typhoeus)
          expect(Crowdkit.config.stack.handlers.last).to eq Faraday::Adapter::Typhoeus
        end
      end
    end

    context "non defaults" do
      shared_examples "overrides" do
        specify "user_agent" do
          expect(client.config.user_agent).to eq("Foo")
        end

        specify "nested connection_options" do
          expect(client.config.connection_options).to include({
            headers: {
              accept: "application/json",
              user_agent: "Foo"
            }
          })
        end

        specify "logging" do
          expect(client.config.stack.handlers.first).to eq(Faraday::Response::Logger)
        end
      end

      context "global" do
        let(:client) do
          Crowdkit
        end

        before(:each) do
          Crowdkit.reset!
          Crowdkit.configure(debug: true, user_agent: "Foo")
        end
      end

      context "block" do
        let(:client) do
          Crowdkit.new do |c|
            c.debug = true
            c.user_agent = "Foo"
          end
        end

        it_behaves_like "overrides"
      end

      context "opts" do
        let(:client) do
          Crowdkit.new({debug: true, user_agent: "Foo"})
        end

        it_behaves_like "overrides"
      end

      context "env" do
        around(:each) do |example|
          Crowdkit.reset!
          ENV["CROWDKIT_USER_AGENT"] = "Foo"
          ENV["CROWDKIT_DEBUG"] = "true"
          example.run
          ENV["CROWDKIT_USER_AGENT"] = nil
          ENV["CROWDKIT_DEBUG"] = nil
        end

        let(:client) do
          Crowdkit
        end

        it_behaves_like "overrides"
      end
    end
  end

  context "client proxy" do
    context "unconfigured" do
      specify "exception" do
        expect { Crowdkit.do_get("/foo") }.to raise_error Crowdkit::Unauthorized
      end
    end
  end

  context "rate limits" do
    let!(:soft_request_stub) { stub_get(/jobs\/soft/).to_return(body: 'rate limit exceeded', status: 429) }
    let!(:hard_request_stub) { stub_get(/jobs\/hard/).to_return(body: 'rate limit exceeded', status: 403) }

    it "retries once on hard" do
      expect { client.jobs.get("hard") }.to raise_error(Crowdkit::Forbidden)
    end

    it "raises error on soft" do
      expect { client.jobs.get("soft") }.to raise_error(Crowdkit::TooManyRequests)
      expect(soft_request_stub).to have_been_requested.times(2)
    end
  end
end
